/******************************************************************************
 * LC7981/HD61830 Graphics +Text LCD Driver
 *
 * Configured for the Samsung LJ41-00192B 160x80 monochrome graphics lcd.
 *
 * Written by Vanya A. Sergeev - <vsergeev@gmail.com>                         
 *
 * February 16, 2007                                                       
 * gendert Juni+Juli 2009 Wolfgang 
***************************************************************************/

/* lc7981.c: Source code for the LC7981/HD61830 graphics lcd driver.
 * The hardware port defines can be found in lc7981.h. */

#define LCD_WIDTH   160
#define LCD_HEIGHT  80

/* Convenient macros to toggle RS, RW, and E control pins, bequeme Makros */ 
#define lcd_rs_high() P6OUT |= BIT6;
#define lcd_rs_low()  P6OUT &=~BIT6;  

#define lcd_rw_high() P6OUT |= BIT5; 
#define lcd_rw_low()  P6OUT &=~BIT5; 

#define lcd_enable_high()  P6OUT |= BIT3; 
#define lcd_enable_low()   P6OUT &=~BIT3;  

/* All possible instructions for the LCD Instruction Register */ //alle mglichen Befehle des Befehlsregisters
#define LCD_CMD_MODE                0x00
#define LCD_CMD_CHAR_PITCH          0x01  //Zeichenfeld
#define LCD_CMD_NUM_CHARS           0x02
#define LCD_CMD_TIME_DIVISION       0x03
#define LCD_CMD_CURSOR_POS          0x04

#define LCD_CMD_DISPLAY_START_LA    0x08
#define LCD_CMD_DISPLAY_START_HA    0x09
#define LCD_CMD_CURSOR_LA           0x0A
#define LCD_CMD_CURSOR_HA           0x0B

#define LCD_CMD_WRITE_DATA          0x0C
#define LCD_CMD_READ_DATA           0x0D

#define LCD_CMD_CLEAR_BIT           0x0E
#define LCD_CMD_SET_BIT             0x0F

/* Bits of the LCD Mode Control Register (DB5-DB0) */
#define LCD_MODE_ON_OFF             32
#define LCD_MODE_MASTER_SLAVE       16
#define LCD_MODE_BLINK               8           //Zeichen blinkt Cursor AUS
#define CURSOR_BLINKT               12          //Cursor EIN und blinkt
#define LCD_MODE_CURSOR              4     //Cursor EIN
#define LCD_MODE_MODE                2     //grafische Betriebsart
#define LCD_MODE_EXTERNAL            1

/* Possible settings of the character pitch register */

#define LCD_CHAR_PITCH_HP_6 0x05
#define LCD_CHAR_PITCH_HP_7 0x06
#define LCD_CHAR_PITCH_HP_8 0x07
#define LCD_CHAR_PITCH_8x8  0x77    //Zeichenfeld: Bit 7-4 --> vertikal, Bit 3-0 --> horizontal
#define LCD_CHAR_PITCH_8x6  0x75    //Zeichenfeld: Bit 7-4 --> vertikal, Bit 3-0 --> horizontal

/* To indicate the state of a pixel */
#define PIXEL_ON    0xFF
#define PIXEL_OFF   0x00

/* Low-level LCD interface functions */
void lcd_strobe_enable(void);
void lcd_write_command(unsigned char command, unsigned char data);
void lcd_wait_busy(void);

/* LCD Graphics interface */
void lcd_graphics_init(void);
void lcd_graphics_clear(void);
void lcd_graphics_move(unsigned short x, unsigned short y);
void lcd_graphics_draw_byte(unsigned char data);
void lcd_graphics_plot_byte(unsigned short x, unsigned short y, unsigned char data);
void lcd_graphics_plot_pixel(unsigned short x, unsigned short y, unsigned char state);

void lcd_print_string(const char *string);
void lcd_set_cursor(unsigned int spalte, unsigned int zeile);

void Pause1(void);
void pausek(void);
void text_init(void);
void text_loeschen(void);

